package com.swissborg.lithium

package strategy

import cats.Monoid
import cats.implicits._

import scala.collection.immutable.SortedSet

class DecisionSpec extends LithiumSpec {
  "StrategyDecision" must {
    "extract the correct nodes from the world view" in {
      forAll { worldView: WorldView =>
        Decision.DownReachable(worldView).nodesToDown should ===(worldView.reachableNodes)
        Decision.DownUnreachable(worldView).nodesToDown should ===(worldView.unreachableNodes)

        Decision.DownIndirectlyConnected(worldView).nodesToDown should
          ===(worldView.indirectlyConnectedNodes)
      }
    }

    "extract the correct nodes from the decision" in {
      forAll { strategyDecision: Decision =>
        strategyDecision match {
          case Decision.DownReachable(nodesToDown) =>
            strategyDecision.nodesToDown should ===(nodesToDown.map(identity[Node]))

          case Decision.DownUnreachable(nodesToDown) =>
            strategyDecision.nodesToDown should ===(nodesToDown.map(identity[Node]))

          case Decision.DownThese(decision1, decision2) =>
            strategyDecision.nodesToDown should ===(decision1.nodesToDown ++ decision2.nodesToDown)

          case Decision.DownIndirectlyConnected(nodesToDown) =>
            strategyDecision.nodesToDown should ===(nodesToDown.map(identity[Node]))

          case Decision.DownAll(nodesToDown) =>
            strategyDecision.nodesToDown should ===(nodesToDown.toSortedSet)

          case Decision.Idle => strategyDecision.nodesToDown.isEmpty shouldBe true
        }
      }
    }

    "correctly combine decisions" in {
      forAll { decisions: List[Decision] =>
        val expectedNodesToDown: SortedSet[Node] =
          SortedSet(decisions.flatMap(_.nodesToDown): _*)
        val combined: SortedSet[Node] =
          decisions.foldRight(Monoid[Decision].empty)(Monoid[Decision].combine).nodesToDown
        combined should contain theSameElementsAs expectedNodesToDown
      }
    }
  }
}
